/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:net.bigeon.gclc.test.utils.WritingRunnable.java
 * Created on: Nov 29, 2016
 */
package net.bigeon.gclc.utils;

/*-
 * #%L
 * Generic Command Ligne console
 * %%
 * Copyright (C) 2014 - 2018 bigeon.fr
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.nio.charset.Charset;
import java.util.ArrayDeque;
import java.util.Deque;
import java.util.logging.Level;
import java.util.logging.Logger;

/** The runnable in charge of writing messages as they are read.
 * <p>
 * Messages are queued to be retrieved latter on.
 *
 * @author Emmanuel Bigeon */
public final class WritingRunnable implements Runnable {

    /** Wait timeout. */
    private static final long   TIMEOUT  = 1000;
    /** Class logger. */
    private static final Logger LOGGER   = Logger
            .getLogger(WritingRunnable.class.getName());
    /** Messages to write. */
    private final Deque<String> messages = new ArrayDeque<>();
    /** Stream to write to. */
    private final OutputStream  outPrint;
    /** The charset. */
    private final Charset       charset;
    /** Runnable state. */
    private boolean             running  = true;

    /** Synchro object. */
    private final Object lock = new Object();

    /** Create the writing runnable.
     *
     * @param outPrint the output to print to
     * @param charset the charset of the stream */
    public WritingRunnable(final OutputStream outPrint, final Charset charset) {
        super();
        this.outPrint = outPrint;
        this.charset = charset;
    }

    /** Add a message in the queue.
     *
     * @param message the message
     * @throws IOException if the pipe is closed */
    public void addMessage(final String message) throws IOException {
        synchronized (lock) {
            if (!running) {
                throw new IOException("Closed pipe"); //$NON-NLS-1$
            }
            messages.offer(message);
            lock.notifyAll();
        }
    }

    /** Test if the message is running.
     *
     * @return the running */
    public boolean isRunning() {
        synchronized (lock) {
            return running;
        }
    }

    /* (non-Javadoc)
     * @see java.lang.Runnable#run() */
    @Override
    public void run() {
        while (running) {
            synchronized (lock) {
                waitNextMessage();
                if (!running) {
                    return;
                }
                writeMessage();
            }
        }

    }

    /** Set the running status.
     *
     * @param running the running to set */
    public void setRunning(final boolean running) {
        synchronized (lock) {
            this.running = running;
        }
    }

    /** Wait for next message. */
    private void waitNextMessage() {
        try {
            synchronized (lock) {
                while (running && messages.isEmpty()) {
                    lock.wait(TIMEOUT);
                }
            }
        } catch (final InterruptedException e) {
            synchronized (lock) {
                if (running) {
                    LOGGER.log(Level.SEVERE, "Thread interruption exception.", e); //$NON-NLS-1$
                }
            }
            Thread.currentThread().interrupt();
        }
    }

    /** Write next message to output. */
    private void writeMessage() {
        final String message = messages.poll();
        final ByteBuffer buff = charset.encode(message + System.lineSeparator());
        if (buff.hasArray()) {
            try {
                outPrint.write(buff.array());
            } catch (final IOException e) {
                LOGGER.log(Level.SEVERE, "Unable to write to stream", //$NON-NLS-1$
                        e);
            }
        }
    }
}
